package edu;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import owrti.util.CDRCoderEncoder;
import edu.rti.*;

public class RectElement extends RTIObject implements Element {
  Rectangle bounds;
  int tX=0;
  int tY=0;
  Color color;
  int deltaX=5, deltaY=5, deltaW=5, deltaH=5, deltaTX=5, deltaTY=5;

  public RectElement() { this(10,10,10,10,Color.red,-1,0,0); }

  RectElement(int _objectID) { this(0,0,0,0,Color.red,_objectID,0,0);   }

  RectElement (int x, int y, int w, int h, Color c) { this(x,y,w,h,c,-1,0,0);     }

  RectElement (int x, int y, int w, int h, Color c,int _objectID, int _tX, int _tY) {
    bounds = new Rectangle (x, y, w, h);
    color = c; tX=_tX; tY=_tY;
    setInstanceRTIid(_objectID);
    try{ setObjectClassRTIid(RTILookUpTable.getObjectClassRTIid("Rectangle"));}
    catch(ObjectClassNameUnknown ocnu) {}
  }

  public Rectangle getBounds () {  return bounds; }

  /**  */
  public void commitTranslation(int dX, int dY) {
    bounds = new Rectangle (bounds.x + dX, bounds.y + dY, bounds.width, bounds.height);
    tX=tY=0; deltaTX=deltaTY=0; deltaX=deltaY=0;fireUpdate(); 
  }// end of commitTranslation()

  /**  */
  public void updateTranslation(int dX, int dY) {
     deltaTX-=Math.abs(dX-tX);deltaTY-=Math.abs(dY-tY);tX=dX;tY=dY;fireUpdate();
  }// end of commitTranslation()

  public boolean isSelectable() { return isPublishable(); }

  /** */
  public void paint (Graphics g) { 
    Color cc = g.getColor();
    g.setColor (color);
    if ((bounds.width == 1) && (bounds.height == 1))
         g.drawLine (bounds.x+tX, bounds.y+tY, bounds.x, bounds.y);
    else g.drawRect (bounds.x+tX, bounds.y+tY, bounds.width - 1, bounds.height - 1);
    g.setColor(cc);
  }// end of paint()

  public void setCord(int x, int y, int w, int h) {
    deltaX=0;deltaY=0; deltaW=0;deltaH=0;
    deltaW-=Math.abs(w-bounds.width); deltaH-=Math.abs(h-bounds.height);
    bounds = new Rectangle (x, y, w, h);
    if(WBDEBUG.DEBUG1) System.out.println("setCord of RectElement");
    fireUpdate();
  }// end of setCord()  

  UpdateListener reflect=null;

  public void setUpdateListener(UpdateListener lis) { reflect=lis; }
   
  final void fireUpdate() {
   if( reflect!=null ) reflect.updateOccurred(new UpdateEvent(this));
  }// end of fireUpdate()

  /// RTIObject hooks
  public Properties getAttributes(boolean state) {
    Properties ret=new Properties();
    if(state || deltaX<1) { ret.put("x",new String(CDRCoderEncoder.toByte(bounds.x)));deltaX=5; }
    if(state || deltaY<1) { ret.put("y",new String(CDRCoderEncoder.toByte(bounds.y)));deltaY=5; }
    if(state || deltaW<1) { ret.put("width",new String(CDRCoderEncoder.toByte(bounds.width)));deltaW=5; }
    if(state || deltaH<1) { ret.put("height",new String(CDRCoderEncoder.toByte(bounds.height))); deltaH=5; }
    if(state || deltaTX<1) { ret.put("transx",new String(CDRCoderEncoder.toByte(tX)));deltaTX=5; }
    if(state || deltaTY<1) { ret.put("transy",new String(CDRCoderEncoder.toByte(tY)));deltaTY=5; }
    if(WBDEBUG.DEBUG1) System.out.println(" CHANGED!!! getAttributes() ");
    return ret;
  }// end of getAttributes() 

  public void setAttributes(Properties p) {
    if(p!=null) {
      String value = (String) p.get("x");
      if(value!=null) bounds.x = CDRCoderEncoder.extractInt(value.getBytes());
      value = (String) p.get("y");
      if(value!=null) bounds.y = CDRCoderEncoder.extractInt(value.getBytes());
      value = (String) p.get("width");
      if(value!=null) bounds.width = CDRCoderEncoder.extractInt(value.getBytes());
      value = (String) p.get("height");
      if(value!=null) bounds.height = CDRCoderEncoder.extractInt(value.getBytes());
      value = (String) p.get("transx");
      if(value!=null) tX = CDRCoderEncoder.extractInt(value.getBytes());
      value = (String) p.get("transy");
      if(value!=null) tY = CDRCoderEncoder.extractInt(value.getBytes());
      fireUpdate();    
      if(WBDEBUG.DEBUG1) System.out.println(" CHANGED!!! setAttributes() ");
    }// end of if
  }// end of setAttributes() 

  public void reflectAttributes(Vector p) {
    if(p!=null&&p.size()>0) {
      if(p.contains("x")) deltaX=0; 
      if(p.contains("y")) deltaY=0; 
      if(p.contains("width")) deltaW=0;
      if(p.contains("height")) deltaH=0;
      if(p.contains("transx")) deltaTX=0;
      if(p.contains("transy")) deltaTY=0;
      deltaX=0;deltaY=0;deltaW=0;deltaH=0;deltaTX=0;deltaTY=0;
      fireUpdate();    
    }// end of if
  }// end of reflectAttributes() 

}// end of class RectElement
