package uci.edu;
import java.io.*;
import org.omg.CORBA.*;

import RTIcap.*;
import owrti.util.CDRCoderEncoder;
import uci.edu.rti.*;

/**
*
* This is callback object to receive RTI messages.
* We used the following callback messages for this demo program:
*
* <p>
* <ol>
* <p> <li> discoverObjectInstance
* <p> <li> removeObjectInstance[With Time]
* <p> <li> reflectAttributeValues[WithTime]
* <p> <li> provideAttributeValueUpdate
* <p> <li> turnUpdatesOnForObjectInstance
* <p> <li> turnUpdatesOffForObjectInstance
* <p> <li> timeAdcanceGrant
* <p> <li> timeRegulationEnabled
* <p> <li> timeConstrainedEnabled
* </ol>
*
* <p>
* @date May 11, 1998 ( Updated May 11, 1998 )
* @author H. Timucin Ozdemir
*
*/     
public class WBFedAmb extends BaseFedAmb 
{
    RTIObjectsContainer  boss;

    public WBFedAmb( RTIObjectsContainer father) {  
        boss = father; 
    }// end of CONSTRUCTOR

    /**
    * Add this new discovered object to the EntityManager.
    */
    public synchronized void 
    discoverObjectInstance(int theObject, short theObjectClass)
        throws RTIcap.CouldNotDiscover, RTIcap.ObjectClassNotKnown, 
               RTIcap.InvalidFederationTime, RTIcap.FederateInternalError
    { 
        if(boss!=null) boss.addObjectInstance(theObject,theObjectClass);
        
    }// end of discoverObjectInstance()

    /**    
    * Update attributes.
    */
    public synchronized void 
    reflectAttributeValuesWithTime(int theObject, 
               RTIcap.AttributeHandleValuePair[] theAttributes, double theTime, 
               String theTag, RTIcap.EventRetractionHandle theHandle)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.FederateOwnsAttributes, RTIcap.InvalidFederationTime, 
               RTIcap.FederateInternalError
    { 
        reflectAttributeValues(theObject,theAttributes,theTag); 
    }// end of reflectAttributeValuesWithTime()

    /**
    *
    * Find the objects in the EntityManager and updates their received
    * attributes. Note that, CDR provided us the necessary decode/encode
    * mechanism for various data types.
    *
    */
    public void 
    reflectAttributeValues(int theObject, 
               RTIcap.AttributeHandleValuePair[] theAttributes, String theTag)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.FederateOwnsAttributes, RTIcap.FederateInternalError
    { 
        RTIObject c = boss.getObjectInstance(theObject);
        if( c != null ) c.addAttributes(theAttributes);
        else if(WBDEBUG.DEBUG1) System.out.println(" ***I do not know this object ("+theObject+")***");
    }// end of reflectAttributeValues()

    /**
    */
    public synchronized void 
    receiveInteractionWithTime(short theInteraction, 
               RTIcap.ParameterHandleValuePair[] theParameters, double theTime, 
               String theTag, RTIcap.EventRetractionHandle theHandle)
        throws RTIcap.InteractionClassNotKnown, 
               RTIcap.InteractionParameterNotKnown, 
               RTIcap.InvalidFederationTime, RTIcap.FederateInternalError
    {}// end of receiveInteractionWithTime()

    /**
    */
    public synchronized void 
    receiveInteraction(short theInteraction, 
               RTIcap.ParameterHandleValuePair[] theParameters, String theTag)
        throws RTIcap.InteractionClassNotKnown, 
               RTIcap.InteractionParameterNotKnown, RTIcap.FederateInternalError
    {}// end of receiveInteraction()

    /**
    * Remove the corresponding object from EntityManager.
    */
    public synchronized void 
    removeObjectInstanceWithTime(int theObject, double theTime, String theTag, 
               RTIcap.EventRetractionHandle theHandle)
        throws RTIcap.ObjectNotKnown, RTIcap.InvalidFederationTime, 
               RTIcap.FederateInternalError
    {
	 if(boss!=null) boss.removeObjectInstance(theObject);
    }// end of removeObjectInstanceWithTime()

    /**
    * Remove the corresponding object from EntityManager.
    */
    public synchronized void 
    removeObjectInstance(int theObject, String theTag)
        throws RTIcap.ObjectNotKnown, RTIcap.FederateInternalError
    {
	 if(boss!=null) boss.removeObjectInstance(theObject);
    }// end of removeObjectInstanceWithTime()

    /**
    * Ask the corresponding SimCountry object to update its attribute.
    */
    public synchronized void 
    provideAttributeValueUpdate(int theObject, short[] theAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.AttributeNotOwned, RTIcap.FederateInternalError
    {
      if(  boss!=null ) {
        RTIObject c = boss.getObjectInstance(theObject);
        if( c != null ) c.reflectAttributes(theAttributes);
        else if(WBDEBUG.DEBUG1) System.out.println(" ***I do not know this object ("+theObject+")***");
      } else if(WBDEBUG.DEBUG1) System.out.println(" EntityManager is NULL ");
    }// end of provideAttributeValueUpdate()

    /**
    * Tell simulated object to send its updates to RTI.
    */
    public synchronized void 
    turnUpdatesOnForObjectInstance(int theObject, short[] theAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotOwned, 
               RTIcap.FederateInternalError
    { 
      if( boss!=null ) {
        RTIObject c = boss.getObjectInstance(theObject);
        if( c != null ) c.reflectAttributes(theAttributes);
        else if(WBDEBUG.DEBUG1) System.out.println(" ***I do not know this object ("+theObject+")***");
      } else if(WBDEBUG.DEBUG1) System.out.println(" EntityManager is NULL ");

    }// end of turnUpdatesOnForObjectInstance()

    /**
    * Tell simulated object *NOT* to send its updates to RTI.
    */
    public synchronized void 
    turnUpdatesOffForObjectInstance(int theObject, short[] theAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotOwned, 
               RTIcap.FederateInternalError
    { 
        if( boss!=null )
        {
          //ChatBudy c = entManager.getEntity(theObject);
          //if( c != null && c instanceof SimChatBudy ) 
          //  ((SimChatBudy)c).UpdateControls(false,theAttributes); 
        } else if(WBDEBUG.DEBUG1) System.out.println(" EntityManager is NULL ");

    }// end of turnUpdatesOffForObjectInstance()

    /**
    * Change the HelloWorld federation's time. And inform that time is
    * advanced.
    */
    public synchronized void 
    timeAdvanceGrant(double theTime)
        throws RTIcap.InvalidFederationTime, 
               RTIcap.TimeAdvanceWasNotInProgress, 
               RTIcap.FederateInternalError
    { 
        //boss.setCurrentTime( theTime ); 
        //boss.setTimeAdvanceGranted();

    }// end of timeAdvanceGrant()

    /** 
    * Change the HelloWorld federate's time. And inform that time is
    * advanced.
    */
    public synchronized void 
    timeRegulationEnabled(double theTime)
        throws RTIcap.InvalidFederationTime, 
               RTIcap.EnableTimeRegulationWasNotPending, 
               RTIcap.FederateInternalError
    { 
        //boss.setCurrentTime( theTime );
        //boss.setTimeAdvanceGranted();

    }// end of timeRegulationEnabled()

    /**
    * Change the HelloWorld federate's time. And inform that time is
    * advanced.
    */
    public synchronized void 
    timeConstrainedEnabled(double theTime)
        throws RTIcap.InvalidFederationTime, 
               RTIcap.EnableTimeConstrainedWasNotPending, 
               RTIcap.FederateInternalError
    { 
        //boss.setCurrentTime( theTime );
        //boss.setTimeAdvanceGranted();

    }// end of timeConstrainedEnabled()

}// end of WBFedAmb
