package uci.edu.dmi;
import java.util.*;
/**
* @author Hasan Timucin Ozdemir
* @date   March 16, 1999 (Updated on March 16, 1999)
*/
public class GenericMessageBroadcaster {
   private int fNumInvokers;
   private int fPriority;
   private Vector invokerPool = new Vector();
   private Vector invokerPoolB = new Vector();
   private Object invokerPoolLock  = new Object();
   private Object invokerPoolBLock = new Object();

   private long fMaxProcesstime = TEN_SECONDS;
   private Thread fControl = new ControlThread();

   private Vector vGenericMessages = new Vector();

   public static final long TEN_SECONDS = 10000;
   public static final long THIRTY_SECONDS = 30000;
   public static final long SIXTY_SECONDS = 60000;
   public static int SWEEP_RATE = 4000;

   /**  */
   public GenericMessageBroadcaster(int priority, int num) {
     if (priority<Thread.MIN_PRIORITY || priority>Thread.MAX_PRIORITY)
        throw new IllegalArgumentException("Invalid thread priority: "+priority);
     fNumInvokers = num;
     fPriority = priority;
     fControl.start();
   }// end of CONSTRUCTOR

   /** */
   public synchronized void start() {
      Invoker n;
      if (fNumInvokers == 0) return;
      for (int i=fNumInvokers; i>0; i--) {
         n = new Invoker(i, fPriority, this);
         invokerPool.addElement(n);
      }
   }

   public synchronized void stop() {
      Invoker n;
      Enumeration e = invokerPool.elements();
      while (e.hasMoreElements()) {
         n = (Invoker)e.nextElement();
         n.timeToStop();
      }// end of while
      invokerPool.removeAllElements();
   }

   Object vGenericMessageLock = new Object();

   /**
   * @param GenericMessage
   */
   public synchronized void broadcastGenericMessage(GenericMessage gm) {
      if(gm!=null) {
         synchronized(vGenericMessageLock) { vGenericMessages.addElement(gm); };
         notify();
      }// end of if
   }// end of broadcastGenericMessage()

   /**
   *
   */
   public void waitForBroadcast(Invoker n) throws InterruptedException {
      GenericMessage req = null;
      synchronized(this) {
         while (vGenericMessages.size() == 0) { wait(); }
         req = (GenericMessage)vGenericMessages.elementAt(0);
         vGenericMessages.removeElementAt(0);
      }// end of synchronized
      if(n!=null&&req!=null) n.setGenericMessage(req);
   }// end of waitForBroadcast()

   Vector listeners = new Vector ();
   Object listenersLock = new Object();

   public void addListener (Object l) {
    synchronized(listenersLock) { listeners.addElement (l); }
   }

   public void removeListener (Object l) {
    synchronized(listenersLock) { listeners.removeElement (l); }
   }

   public Enumeration getListeners () { return listeners.elements(); }

   public int getNumberOfListeners () {	return listeners.size(); }

   public Object getFirstListener () { return listeners.elementAt(0); }

   /**
   * 
   */
   public int getNumThreads() { return fNumInvokers; }

   /**
   * 
   */
   public void setMaxProcesstime(long t) { fMaxProcesstime = t; }

   /**
   * 
   */
   public long getMaxProcesstime() { return fMaxProcesstime; }

   void checkForTimeout(long curr, long max) {
      Enumeration e = invokerPool.elements();
      while (e.hasMoreElements()) {
         Invoker n = (Invoker)e.nextElement();
         n.checkForTimeout(curr, max);
      }// end of while
   }// end of checkForTimeout()

   class ControlThread extends Thread {

      ControlThread() {
         setPriority(MIN_PRIORITY);
         setDaemon(true);
      }// end of CONSTRUCTOR

      public void run() {
         while (true) {
            try {
               sleep(SWEEP_RATE);
            } catch (InterruptedException ex) {}

            long max = fMaxProcesstime;
            long curr = System.currentTimeMillis();

            checkForTimeout(curr, max);
         }
      }
   }// end of inline class ControlThread

}// end of GenericMessageBroadcaster
