package uci.edu.rti;
import java.util.*;
/**
*
* @author Hasan Timucin Ozdemir
* @date   March 16, 1999 (Updated on March 16, 1999)
*
*/
class ObjRecord
{
  Hashtable attribs = new Hashtable();
  Hashtable attribs2 = new Hashtable();
  short classId = -1;
  String className;

  /**
  * @param _className
  * @param _classId
  */  
  public ObjRecord(String _className, int _classId )
  { className = _className; classId = (short)_classId; }

  /**
  * @return The RTI handle of object class.
  */    
  public short getClassId() { return classId; }

  /**
  * @return The name of object class.
  */  
  public String getClassName() { return className; }

  /**
  * @param attName
  * @param attId
  */  
  public void addAttributeId(String attName, int attId )
  { 
    Integer a = (Integer)attribs.get(attName);
    if(a==null) { attribs.put(attName,new Integer(attId));attribs2.put(new Short((short)attId),attName); }
  }// end of addAttributeId()

  /**
  * @param attName
  * @return RTI Attribute Handle.
  */  
  public short 
  getAttributeId(String attName) throws ObjectClassAttributeUnknown
  { Integer a = (Integer)attribs.get(attName);
    if(a==null) throw new ObjectClassAttributeUnknown();
    return a.shortValue();
  }// end of getAttributeId()

  /**
  * @param attName
  * @return RTI Attribute Handle.
  */  
  public String 
  getAttributeName(short identifier) throws ObjectClassAttributeUnknown
  { String a = (String)attribs2.get(new Short(identifier));
    if(a==null) throw new ObjectClassAttributeUnknown();
    return a;
  }// end of getAttributeName()

  /**
  * @return RTI Attribute Handles.
  */
  public short[] getAttributeRTIHandles() {
    short[] ret = null;
    if(attribs.size()>0) {
      int i=0;
      ret = new short[attribs.size()];
      Enumeration keys = attribs.keys();
      while( keys.hasMoreElements() )
        try
        { ret[i]= getAttributeId((String)keys.nextElement()); i++; }
        catch(ObjectClassAttributeUnknown ocau) {}
      if(i<ret.length) { short[] reta = new short[i]; for(;i>0;i--) reta[i-1]=ret[i]; ret=reta; }
    }// end of if
    return ret;
  }// end of getAttributeRTIHandles()
  
}// end of ObjRecord

/**
*
* @author Hasan Timucin Ozdemir
* @date   March 16, 1999 (Updated on March 16, 1999)
*
*/
class InterRecord
{
  Hashtable params = new Hashtable();
  short  classId = -1;
  String className;

  /**
  * @param _className
  * @param _classId
  */  
  public InterRecord(String _className, int _classId )
  { className = _className; classId = (short)_classId; }

  /**
  * @return The RTI handle of interaction class.
  */    
  public short getClassId() { return classId; }

  /**
  * @return The name of interaction  class.
  */  
  public String getClassName() { return className; }

  /**
  * @param paramName
  * @param paramId
  */  
  public void addParameterId(String paramName, int paramId )
  { Integer a = (Integer)params.get(paramName);
    if(a==null) params.put(paramName,new Integer(paramId));
  }// end of addParameterId()

  /**
  * @param paramName
  * @return RTI Parameter Handle.
  */  
  public int getParameterId(String paramName) throws InteractionClassParameterUnknown
  { Integer a = (Integer)params.get(paramName);
    if(a==null) throw new InteractionClassParameterUnknown();
    return a.intValue();
  }// end of getParameterId()

}// end of InterRecord

/**
* Singleton.
*
* @author Hasan Timucin Ozdemir
* @date   March 16, 1999 (Updated on March 16, 1999)
*
*/
public class RTILookUpTable  
{
  static Hashtable objRecs   = new Hashtable();
  static Hashtable interRecs = new Hashtable();
  static Hashtable objCLSIDRecs = new Hashtable();

  /**
  * @param objectClassName 
  * @see ObjectClassNameUnknown
  */  
  public static synchronized short 
  getObjectClassRTIid(String objectClassName) throws ObjectClassNameUnknown { 
     return getObjectRecord(objectClassName).getClassId();
  }// end of getObjectClassRTIid()

  /**
  * @param objectClassName 
  * @see ObjectClassNameUnknown
  */  
  public static synchronized String 
  getObjectClassName(short identifier) throws ObjectClassNameUnknown {  
    return getObjectRecord(identifier).getClassName();
  }// end of getObjectClassRTIid()

  /**
  * @param objectClassName 
  * @param attributeName 
  * @exception ObjectClassNameUnknown
  * @exception ObjectClassAttributeUnknown
  */  
  public static synchronized short 
  getObjectClassAttributeRTIid(String objectClassName, String attributeName ) 
         throws ObjectClassNameUnknown, ObjectClassAttributeUnknown  {
     return getObjectRecord(objectClassName).getAttributeId(attributeName);
  }// end of getObjectClassAttributeRTIid()

  /**
  * @param classId 
  * @param attributeName 
  * @exception ObjectClassNameUnknown
  * @exception ObjectClassAttributeUnknown
  */  
  public static synchronized short 
  getObjectClassAttributeRTIid(short classId, String attributeName ) 
         throws ObjectClassNameUnknown, ObjectClassAttributeUnknown {
     return getObjectRecord(classId).getAttributeId(attributeName);
  }// end of getObjectClassAttributeRTIid()

  /**
  * @param classId 
  * @param attributeId 
  * @exception ObjectClassNameUnknown
  * @exception ObjectClassAttributeUnknown
  */  
  public static synchronized String 
  getObjectClassAttributeName(short classId, int attributeId ) 
         throws ObjectClassNameUnknown, ObjectClassAttributeUnknown {
     return getObjectRecord(classId).getAttributeName((short)attributeId);
  }// end of getObjectClassAttributeRTIid()

  /**
  * @param objectClassName 
  * @param identifier 
  */  
  public static synchronized void 
  addObjectClass(String objectClassName, short identifier ) 
  { ObjRecord oRec = (ObjRecord)objRecs.get(objectClassName);
    if( oRec==null )
    { oRec = new ObjRecord(objectClassName,identifier);
      objRecs.put(objectClassName,oRec); objCLSIDRecs.put(new Short(identifier),oRec);
    }// end of if
  }// end of addObjectClass()

  /**
  * @param objectClassName
  * @param attributeName 
  * @param identifier 
  */  
  public static synchronized void 
  addObjectClassAttribute(String objectClassName, String attributeName, int identifier )
         throws ObjectClassNameUnknown { 
    getObjectRecord(objectClassName).addAttributeId(attributeName,identifier);
  }// end of addObjectClassAttribute()

  /**
  * @param objectClassName
  */  
  public static synchronized short[] 
  getObjectClassAttributeRTIHandles(String objectClassName) throws ObjectClassNameUnknown {
    return getObjectRecord(objectClassName).getAttributeRTIHandles(); 
  }// end of addObjectClassAttribute()

  /**
  * @param objectClassName 
  * @exception ObjectClassNameUnknown
  */  
  static ObjRecord getObjectRecord(String objectClassName) throws ObjectClassNameUnknown {
    ObjRecord oRec = (ObjRecord)objRecs.get(objectClassName);
    if(oRec==null) throw new ObjectClassNameUnknown(objectClassName);
    return oRec;
  }// end of getObjectRecord()
  
  /**
  * @param classId 
  * @exception ObjectClassNameUnknown
  */  
  static ObjRecord getObjectRecord(short classId) throws ObjectClassNameUnknown {
    ObjRecord oRec = (ObjRecord)objCLSIDRecs.get(new Short(classId));
    if( oRec==null ) throw new ObjectClassNameUnknown();
    return oRec;
  }// end of getObjectRecord()

  ///// Interaction Related Methods

  /**
  * @param interactionClassName 
  * @exception InteractionClassNameUnknown
  */  
  public static synchronized short 
  getInteractionClassRTIid(String interactionClassName) throws InteractionClassNameUnknown
  { InterRecord oRec = (InterRecord)interRecs.get(interactionClassName);
    if( oRec==null ) throw new InteractionClassNameUnknown();
    return oRec.getClassId();
  }// end of getInteractionClassRTIid()

  /**
  * @param interactionClassName
  * @param parameterName 
  * @exception InteractionClassNameUnknown
  * @exception InteractionClassParameterUnknown
  */  
  public static synchronized int 
  getInteractionClassParameterRTIid(String interactionClassName, String parameterName)
         throws InteractionClassNameUnknown, InteractionClassParameterUnknown
  { InterRecord oRec = (InterRecord)interRecs.get(interactionClassName);
    if( oRec==null ) throw new InteractionClassNameUnknown();
    return oRec.getParameterId(parameterName);
  }// end of getInteractionClassParameterRTIid()

  /**
  * @param interactionClassName 
  * @param identifier 
  */  
  public static synchronized void 
  addInteractionClass(String interactionClassName, short identifier ) 
  { InterRecord oRec = (InterRecord)interRecs.get(interactionClassName);
    if( oRec==null ) interRecs.put(interactionClassName, new InterRecord(interactionClassName,identifier));
  }// end of addInteractionClass()

  /**
  * @param interactionClassName
  * @param parameterName 
  * @param identifier 
  * @exception InteractionClassNameUnknown
  */  
  public static synchronized void 
  addInteractionClassAttribute(String interactionClassName, String parameterName, int identifier )
         throws InteractionClassNameUnknown
  { InterRecord oRec = (InterRecord)interRecs.get(interactionClassName);
    if( oRec==null ) throw new InteractionClassNameUnknown();
    oRec.addParameterId(parameterName,identifier);
  }// end of addInteractionClassAttribute()
  
}// end of RTILookUpTable

