package uci.edu.rti;
import java.util.*;
import RTIcap.*;

/**
* @author Hasan Timucin Ozdemir
* @date   March 16, 1999 (Updated on March 16, 1999)
*/
public abstract class RTIObject {
  boolean update = false;
  int     objId = -1;
  short   classId=-1;

  /** Sets publishing. */
  public void    setPublish()   { update=true;  }

  /** Resets publishing. */
  public void    resetPublish() { update=false; }

  /**
  * @return need publishing? 
  */
  public boolean isPublishable() { return update;}

  /**
  * @param objId
  */ 
  public void    setInstanceRTIid(int objId) { this.objId=objId;}

  /**
  * @return RTI Object instance handle.
  */ 
  public int     getInstanceRTIid()          { return objId;}

  /**
  * @param classId
  */ 
  public void    setObjectClassRTIid(short classId) { this.classId=classId;}

  /**
  * @return RTI Object class handle.
  */
  public short   getObjectClassRTIid()              { return classId;}
  
  /**
  * @param theAttributes
  */
  public void    addAttributes(RTIcap.AttributeHandleValuePair[] theAttributes)
  { if( theAttributes!=null && theAttributes.length>0 ) {
     Properties p = new Properties();
     for(int i=0;i<theAttributes.length;i++) 
       p.put(RTILookUpTable.getObjectClassAttributeName(classId,theAttributes[i].handle),new String(theAttributes[i].value));
     setAttributes(p);
    }// end of if
  }// end of  addAttributes()

  /**
  * @return AttributeHandleValuePair
  */
  public RTIcap.AttributeHandleValuePair[] getUpdatedAttributes(boolean state)
  { int i=0; RTIcap.AttributeHandleValuePair[] ret = null;
    Properties p = getAttributes(state);
    if(p!=null)
    { Enumeration e = p.keys();  
      ret = new RTIcap.AttributeHandleValuePair[p.size()];
      while(e.hasMoreElements())
      { String aKey = (String) e.nextElement();
        try
        { if(aKey!=null) { String value=(String)p.get(aKey); 
           if(value!=null) ret[i++]=new RTIcap.AttributeHandleValuePair(RTILookUpTable.getObjectClassAttributeRTIid(classId,aKey),value.getBytes());
          }
        } catch(ObjectClassNameUnknown e1) {}
          catch(ObjectClassAttributeUnknown e1) {}
      }//end of while
    }// end of if
    return ret;
  }// end of getUpdatedAttributes()
  
  /**
  * @param theAttributes
  */
  public void    reflectAttributes(short[] theAttributes)
  { if( theAttributes!=null && theAttributes.length>0 ) {
     Vector p = new Vector();
     for(int i=0;i<theAttributes.length;i++) 
       p.addElement(RTILookUpTable.getObjectClassAttributeName(classId,theAttributes[i]));
     reflectAttributes(p);
    }// end of if
  }// end of  addAttributes()

  /**
  * @return <attr_name><value_byte> pairs.
  */
  public abstract Properties getAttributes(boolean state);

  /**
  * @param props <attr_name><value_byte> pairs.
  */
  public abstract void setAttributes(Properties props);

  /**
  * @param props <attr_name><value_byte> pairs.
  */
  public abstract void reflectAttributes(Vector props);

}// end of RTIObject

