// File: CmdCreateNode1.java
// Classes: CmdCreateNode
// Original Author: ashankar@syr.edu

package uci.gef;

import java.util.*;
import uci.edu.*;
import uci.gef.demo.*;
import uci.graph.*;

/** Editor Cmd to create a new FigNode on a new NetNode. When
 *  this Cmd is executed it makes the new objects as per its
 *  arguments, and then it sets the global next mode to ModePlace so
 *  that the user can place the new node in any editor window.
 *
 * @see ModePlace
 * @see NetNode
 * @see FigNode */

public class CmdCreateNode1 extends CmdCreateNode {
	ElementsContainer contents; //to store the entity manager and the new element
 public CmdCreateNode1(Hashtable args, String name) {
    super(args, name);
  }

  /** Construct a new Cmd with the given classes for the NetNode
   *  and its FigNode. */
  public CmdCreateNode1(Class nodeClass, String name) {
    this(new Hashtable(), name);
    setArg("className", nodeClass);
  }

  /** Construct a new Cmd with the given classes for the NetNode
   *  and its FigNode, and set the global sticky mode boolean to
   *  the given value. This allows the user to place several nodes
   *  rapidly.  */
  public CmdCreateNode1(Class nodeClass, boolean sticky, String name) {
    this(nodeClass, name);
    setArg("shouldBeSticky", sticky ? Boolean.TRUE : Boolean.FALSE);
  }

  ////////////////////////////////////////////////////////////////
  // Cmd API
   public void StoreElementsContainer(ElementsContainer cont)
   {
   	System.out.println(" StoreElementsContainer () of CmdCreateNode1 I know what to do I guess now"+cont);

	contents = cont;
   }
  /** Actually instanciate the NetNode and FigNode objects and
   * set the global next mode to ModePlace */
  public void doIt() {
    Editor ce = Globals.curEditor();
    GraphModel gm = ce.getGraphModel();
    if (!(gm instanceof MutableGraphModel)) return;

	System.out.println(" doit () of CmdCreateNode1");
    ModePlace1 placeMode = new ModePlace1(this);
	placeMode.store(contents);
    Object shouldBeSticky = getArg("shouldBeSticky");
    Globals.mode(placeMode, shouldBeSticky == Boolean.TRUE);
	
  }
} /* end class CmdCreateNode */
