//Author : Anusha Shankar

package uci.gef;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import uci.edu.*;



public class ModeSelect1 extends ModeSelect {

  // instance variables

  /** If the user drags a selection rectangle, this is the first corner. */
  private Point	_selectAnchor  = new Point(0, 0);

  /** This is the seclection rectangle. */
  private Rectangle _selectRect = new Rectangle(0,0,0,0);

  /** True when the selection rectangle should be paintn. */
  private boolean _showSelectRect = false;

  /** True when the user holds the shift key to toggle selections. */
  private boolean _toggleSelection = false;

  ////////////////////////////////////////////////////////////////
  // constructors and related methods

  /** Construct a new ModeSelect with the given parent. */
  public ModeSelect1(Editor par) { super(par); }

  /** Construct a new ModeSelect instance. Its parent must be set
   *  before this instance can be used.  */
  public ModeSelect1() { }

   
  ////////////////////////////////////////////////////////////////
  // event handlers

  /** Handle mouse down events by preparing for a drag. If the mouse
   *  down event happens on a handle or an already selected object, and
   *  the shift key is not down, then go to ModeModify. If the mouse
   *  down event happens on a port, to to ModeCreateEdge.   */
  public void mousePressed(MouseEvent me) {
    if (me.isConsumed()) {
      //System.out.println("ModeSelect consumed");
      return;
    }
	System.out.println(" mousePressed(MouseEvent me) of ModeSelect1 entered");
    if (me.getModifiers() == InputEvent.BUTTON3_MASK) { return; }
    if (me.isControlDown()) { gotoBroomMode(me); return; }
    if (me.isAltDown()) return;
    int x = me.getX(), y = me.getY();

    _selectAnchor = new Point(x, y);
    _selectRect.setBounds(x, y, 0, 0);
    _toggleSelection = me.isShiftDown();
    SelectionManager sm = _editor.getSelectionManager();
    Rectangle hitRect = new Rectangle(x - 4, y - 4, 8, 8);

    /* Check if multiple things are selected and user clicked one of them. */
    Fig underMouse = _editor.hit(_selectAnchor);
    if (underMouse == null && !sm.hit(hitRect)) return;

    if (underMouse != null) {
      if (_toggleSelection) sm.toggle(underMouse);
      else if (!sm.containsFig(underMouse)) sm.select(underMouse);
    }
    if (sm.hit(hitRect)) {
      //System.out.println("gotoModifyMode");
      gotoModifyMode(me);
    }
    me.consume();
  }

  /** On mouse dragging, stretch the selection rectangle. */
  public void mouseDragged(MouseEvent me) {
    int x = me.getX(), y = me.getY();int bound_x = Math.min(_selectAnchor.x, x);
    int bound_y = Math.min(_selectAnchor.y, y);
    int bound_w = Math.max(_selectAnchor.x, x) - bound_x;
    int bound_h = Math.max(_selectAnchor.y, y) - bound_y;
    Rectangle hitRect = new Rectangle(x - 4, y - 4, 8, 8);
    _showSelectRect = true;
    Enumeration figs = _editor.figs();
    while (figs.hasMoreElements()) {
      Fig f = (Fig) figs.nextElement();
      if ((!_toggleSelection && _selectRect.isEmpty() && f.hit(hitRect)) ||
	  (!_selectRect.isEmpty() && f.within(_selectRect))) {
	    System.out.println(" \n hey I am in MOUSEDragged(MouseEvent me) of ModeSelect1 entered"+f +"\n\n");
		FigNode1 fign1 = (FigNode1)f;
		NodeElement ne =(NodeElement) fign1.getelement();
		_editor.damaged(_selectRect);
		_editor.damaged(f);
		_selectRect.setBounds(bound_x, bound_y, bound_w, bound_h);
		_editor.damageAll();
		ne.setCord(bound_x,bound_y,bound_w,bound_h);
		ne.commitTranslation(bound_x,bound_y); 
      }
    }
       
    me.consume();
  }

  /** On mouse up, select or toggle the selection of items under the
   *  mouse or in the selection rectangle. */
  public void mouseReleased(MouseEvent me) {
 

    if (me.isConsumed()) return;
    int x = me.getX(), y = me.getY();
    _showSelectRect = false;
    Vector selectList = new Vector();

    Rectangle hitRect = new Rectangle(x - 4, y - 4, 8, 8);
    Enumeration figs = _editor.figs();
    while (figs.hasMoreElements()) {
      Fig f = (Fig) figs.nextElement();
      if ((!_toggleSelection && _selectRect.isEmpty() && f.hit(hitRect)) ||
	  (!_selectRect.isEmpty() && f.within(_selectRect))) {
	selectList.addElement(f);
      }
    }
 	System.out.println(" U R IN MOUSE Released(MouseEvent me) of ModeSelect1 entered"+selectList+ " selection list\n\n");
    if (!_selectRect.isEmpty() && selectList.isEmpty()) {
       figs = _editor.figs();
       while (figs.hasMoreElements()) {
	 Fig f = (Fig) figs.nextElement();
	 if (f.intersects(_selectRect)) selectList.addElement(f);
       }
    }
    if (_toggleSelection) _editor.getSelectionManager().toggle(selectList);
    else _editor.getSelectionManager().select(selectList);
	System.out.println(" U R IN MOUSE Released(MouseEvent me) of ModeSelect1 entered"+hitRect);

 //   _selectRect.setBounds(bound_x, bound_y, bound_w, bound_h);
    _selectRect.grow(1,1); /* make sure it is not empty for redraw */
    _editor.damaged(_selectRect);
    if (me.getModifiers() == InputEvent.BUTTON3_MASK) return;
    me.consume();
  }


  
  ////////////////////////////////////////////////////////////////
  // methods related to transitions among modes

  /** Set the Editor's Mode to ModeModify.  Needs-More-Work: This
   *  should not be in ModeSelect, I wanted to move it to ModeModify,
   *  but it is too tighly integrated with ModeSelect. */
  protected void gotoModifyMode(MouseEvent me) {
    Mode nextMode = new ModeModify(_editor);
    _editor.mode(nextMode);
    nextMode.mousePressed(me);
  }

  protected void gotoBroomMode(MouseEvent me) {
    Mode nextMode = new ModeBroom(_editor);
    _editor.mode(nextMode);
    nextMode.mousePressed(me);
  }


  static final long serialVersionUID = -8847591655429149135L;
} /* end class ModeSelect1 */

